package com.mortals.xhx.module.dept.service.impl;

import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.MatterHtmlParseUtil;
import com.mortals.xhx.module.dept.dao.DeptDao;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.model.vo.DeptVo;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matters.model.MattersDeptEntity;
import com.mortals.xhx.module.matters.model.MattersDeptQuery;
import com.mortals.xhx.module.matters.service.MattersDeptService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * DeptService
 * 部门 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("deptService")
public class DeptServiceImpl extends AbstractCRUDCacheServiceImpl<DeptDao, DeptEntity, Long> implements DeptService {

    @Autowired
    private SiteService siteService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private MattersDeptService mattersDeptService;
//    @Autowired
//    private WindowService windowService;
//    @Autowired
//    private BusinessService businessService;
//    @Autowired
//    private SiteBusinessService siteBusinessService;
//    @Autowired
//    private WindowBusinessService windowBusinessService;


    @Override
    protected String getExtKey(DeptEntity data) {
        return data.getDeptNumber();
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void validData(DeptEntity entity, Context context) throws AppException {
        super.validData(entity, context);
        //校验部门编码是否重复
        DeptEntity extCache = this.getExtCache(entity.getDeptNumber());
        if (!ObjectUtils.isEmpty(extCache) && SourceEnum.自定义.getValue() == entity.getSource()) {
            throw new AppException("部门编码重复!deptCode:" + extCache.getDeptNumber());
        }
    }

    @Override
    public void syncDept(String areaCode, Context context) {
        List<MattersDeptEntity> deptList = mattersDeptService.find(new MattersDeptQuery());
        deptList.forEach(dept -> {
            //根据区域查询当前区域下存在的站点
            List<SiteEntity> siteEntityList = siteService.getFlatSitesByAreaCode(dept.getAreaCode(), context);
            siteEntityList.forEach(site -> {
                //根据站点添加部门信息，部门编号存在时候不添加
                DeptEntity deptExistEntity = deptService.selectOne(new DeptQuery().siteId(site.getId()).deptNumber(dept.getDeptCode()), context);
                if (ObjectUtils.isEmpty(deptExistEntity)) {
                    DeptEntity deptEntity = new DeptEntity();
                    deptEntity.initAttrValue();
                    deptEntity.setDeptNumber(dept.getDeptCode());
                    deptEntity.setSiteId(site.getId());
                    deptEntity.setName(dept.getName());
                    deptEntity.setCreateTime(new Date());
                    deptEntity.setCreateUserId(1L);
                    deptService.save(deptEntity, context);
                }
            });
        });
    }

    @Override
    public Rest<String> syncDeptBySiteId(SiteEntity siteEntity, Context context) {
        String url = GlobalSysInfo.getParamValue(Constant.GOV_DEPT_URL, "http://www.sczwfw.gov.cn/jiq/front/channel/deptSwitch");
        Long siteId = siteEntity.getId();
        String areaCode = siteEntity.getAreaCode();
        Map<String, String> params = new HashMap<>();
        params.put("areaCode", areaCode);
        Rest<Map<String, String>> rest = MatterHtmlParseUtil.syncDeptBySiteId(params, url);
        if (rest.getCode() == YesNoEnum.YES.getValue() && rest.getData().keySet().size() > 0) {
            //删除部门中属于政务部门的
            // deptService.deleteGovBySiteId(siteId, context);
            int sortN = 1;
            Map<String, String> data = rest.getData();
            for (Map.Entry<String, String> item : data.entrySet()) {
                String deptCode = item.getKey();
                String deptName = item.getValue();
                DeptEntity deptEntity = deptService.selectOne(new DeptQuery().siteId(siteId).deptNumber(deptCode), context);
                if (ObjectUtils.isEmpty(deptEntity)) {
                    deptEntity = new DeptEntity();
                    deptEntity.initAttrValue();
                    deptEntity.setDeptNumber(deptCode);
                    deptEntity.setSiteId(siteId);
                    deptEntity.setName(deptName);
                    deptEntity.setSort(sortN);
                    deptEntity.setSource(SourceEnum.政务网.getValue());
                    deptEntity.setCreateTime(new Date());
                    deptEntity.setCreateUserId(1L);
                    deptService.save(deptEntity, context);
                } else {
                    //更新
                    deptEntity.setName(deptName);
                    deptEntity.setSource(SourceEnum.政务网.getValue());
                    deptEntity.setUpdateTime(new Date());
                    deptEntity.setUpdateUserId(1L);
                    deptService.update(deptEntity, context);
                }
                sortN++;
            }
        } else {
            return Rest.fail(rest.getMsg());
        }
        return Rest.ok("当前站点同步添加部门成功！");
    }

    @Override
    public Rest<Map<String, List<DeptVo>>> getBusinessByDept(DeptQuery deptQuery, Context context) {

        List<DeptVo> businessDeptList = this.dao.getBusinessByDept(deptQuery);
        Map<String, List<DeptVo>> collect = businessDeptList.parallelStream().collect(Collectors.groupingBy(x -> x.getId().toString()));

        return Rest.ok(collect);


        /*  //查询部门窗口
        List<WindowEntity> windowEntities = windowService.find(new WindowQuery().deptId(deptQuery.getId()));
        //查询窗口业务
        WindowBusinessQuery windowBusinessQuery = new WindowBusinessQuery();
        List<Long> windowList = windowEntities.stream().map(WindowEntity::getId).collect(Collectors.toList());
        if (ObjectUtils.isEmpty(windowList)) {
            return Rest.ok("窗口查询结果为空", Collections.EMPTY_LIST);
        }
        windowBusinessQuery.setWindowIdList(windowList);
        List<WindowBusinessEntity> windowBusinessEntities = windowBusinessService.find(windowBusinessQuery);

        SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
        List<Long> siteBusinessList = windowBusinessEntities.stream().map(WindowBusinessEntity::getSiteBusinessId).collect(Collectors.toList());
        if (ObjectUtils.isEmpty(siteBusinessList)) {
            return Rest.ok("站点业务查询结果为空", Collections.EMPTY_LIST);
        }
        siteBusinessQuery.setBusinessIdList(siteBusinessList);
        List<SiteBusinessEntity> siteBusinessEntities = siteBusinessService.find(siteBusinessQuery);
        BusinessQuery businessQuery = new BusinessQuery();
        List<Long> businessList = siteBusinessEntities.stream().map(SiteBusinessEntity::getBusinessId).collect(Collectors.toList());
        if (ObjectUtils.isEmpty(businessList)) {
            return Rest.ok("业务查询结果为空", Collections.EMPTY_LIST);
        }
        businessQuery.setIdList(businessList);
        List<BusinessEntity> businessEntities = businessService.find(businessQuery);*/
        //return Rest.ok(businessEntities);
    }

    @Override
    public void deleteGovBySiteId(Long siteId, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("siteId", siteId);
        condition.put("source", SourceEnum.政务网.getValue());
        this.dao.delete(condition);
    }

    /**
     * @param deptQuery
     * @param context
     * @return
     */
    @Override
    public Rest<Map<String, List<DeptVo>>> getDeptListByBusiness(DeptQuery deptQuery, Context context) {
        List<DeptVo> deptListByBusiness = this.dao.getDeptListByBusiness(deptQuery);
        Map<String, List<DeptVo>> collect = deptListByBusiness.parallelStream().collect(Collectors.groupingBy(x -> x.getSiteBusinessId().toString()));
        return Rest.ok(collect);
    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
//    @Override
//    protected void updateAfter(DeptEntity entity, Context context) throws AppException {
//        List<WindowEntity> windowEntityList = windowService.find(new WindowQuery().deptId(entity.getId()));
//        windowEntityList.forEach(item -> {
//            item.setDeptName(entity.getName());
//        });
//        windowService.update(windowEntityList, context);
//        super.updateAfter(entity, context);
//    }
}