package com.mortals.xhx.module.window.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.framework.util.HttpUtil;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.business.model.BusinessQuery;
import com.mortals.xhx.module.business.service.BusinessService;
import com.mortals.xhx.module.site.model.SiteBusinessEntity;
import com.mortals.xhx.module.site.model.SiteBusinessQuery;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteBusinessService;
import com.mortals.xhx.module.window.model.*;
import com.mortals.xhx.module.window.service.WindowBusinessService;
import com.mortals.xhx.module.window.service.WindowMatterService;
import com.mortals.xhx.module.workman.model.WorkmanEntity;
import com.mortals.xhx.module.workman.model.WorkmanQuery;
import com.mortals.xhx.module.workman.service.WorkmanService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.window.dao.WindowDao;
import com.mortals.xhx.module.window.service.WindowService;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.framework.util.HttpUtil.HEADER_CONTENT_TYPE;
import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_PHP_HTTP_URL;

/**
 * WindowService
 * 站点部门窗口 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("windowService")
@Slf4j
public class WindowServiceImpl extends AbstractCRUDCacheServiceImpl<WindowDao, WindowEntity, Long> implements WindowService {

    @Autowired
    private WorkmanService workmanService;

    @Autowired
    private WindowBusinessService windowBusinessService;
    @Autowired
    private WindowMatterService windowMatterService;

    @Autowired
    private BusinessService businessService;


    @Override
    protected void findAfter(WindowEntity params, PageInfo pageInfo, Context context, List<WindowEntity> list) throws AppException {
        Map<Long, List<WorkmanEntity>> workmanCollect = workmanService.find(new WorkmanQuery()).stream().filter(f -> f.getWindowId() != null).collect(Collectors.groupingBy(x -> x.getWindowId()));
        Map<Long, List<WindowBusinessEntity>> windowBusinessCollect = windowBusinessService.find(new WindowBusinessQuery())
                .stream().filter(f -> f.getWindowId() != null).collect(Collectors.groupingBy(x -> x.getWindowId()));

        Map<Long, List<WindowMatterEntity>> windowMatterCollect = windowMatterService.find(new WindowMatterQuery())
                .stream().filter(f -> f.getWindowId() != null).collect(Collectors.groupingBy(x -> x.getWindowId()));

        list.stream().peek(item -> {
            if (!ObjectUtils.isEmpty(workmanCollect.get(item.getId()))) {
                item.setWorkmanList(workmanCollect.get(item.getId()));
            }
            if (!ObjectUtils.isEmpty(windowBusinessCollect.get(item.getId()))) {
                item.setWindowBusinessList(windowBusinessCollect.get(item.getId()));
            }

            if (!ObjectUtils.isEmpty(windowMatterCollect.get(item.getId()))) {
                item.setWindowMatterList(windowMatterCollect.get(item.getId()));
            }

        }).count();
        super.findAfter(params, pageInfo, context, list);
    }


    @Override
    public void addBusinessToWindow(String businessIds, Long windowId, Context context) {
        if (ObjectUtils.isEmpty(windowId)) {
            throw new AppException("请选择对应窗口");
        }
        List<Long> businessIdList = new ArrayList<>();
        if (!ObjectUtils.isEmpty(businessIds)) {
            businessIdList = Arrays.asList(businessIds.split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
        }
        WindowBusinessQuery windowBusinessQuery = new WindowBusinessQuery();
        //windowBusinessQuery.setSiteBusinessIdList(businessIdList);

        //先删除后再新增
        windowBusinessQuery.setWindowId(windowId);
        Long[] ids = windowBusinessService.find(windowBusinessQuery).stream()
                .map(WindowBusinessEntity::getId).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(ids)) {
            windowBusinessService.remove(ids, context);
        }

        WindowEntity windowEntity = this.get(windowId, context);
        List<WindowBusinessEntity> windowBusinessEntities = businessIdList.stream().map(id -> {
            WindowBusinessEntity windowBusinessEntity = new WindowBusinessEntity();
            windowBusinessEntity.initAttrValue();
            if (!ObjectUtils.isEmpty(windowEntity)) {
                windowBusinessEntity.setWindowId(windowId);
                windowBusinessEntity.setWindowName(windowEntity.getName());
            } else {
                return null;
            }
            BusinessEntity businessEntity = businessService.get(id, context);
            if (!ObjectUtils.isEmpty(businessEntity)) {
                windowBusinessEntity.setSiteBusinessId(id);
                windowBusinessEntity.setBusinessName(businessEntity.getName());
            } else {
                return null;
            }
            return windowBusinessEntity;
        }).filter(f -> f != null).collect(Collectors.toList());
        windowBusinessService.save(windowBusinessEntities, context);
    }


    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        if (!ObjectUtils.isEmpty(ids)) {
            //级联删除窗口业务
            WindowBusinessQuery windowBusinessQuery = new WindowBusinessQuery();
            windowBusinessQuery.setWindowIdList(Arrays.asList(ids));
            List<WindowBusinessEntity> windowBusinessEntities = windowBusinessService.find(windowBusinessQuery);
            log.info("remove windowBusinessEntities size:{}",windowBusinessEntities.size());
            if (!ObjectUtils.isEmpty(windowBusinessEntities)) {
                Long[] idList = windowBusinessEntities.stream().map(item -> item.getId()).toArray(Long[]::new);
                //windowBusinessService.removeList(windowBusinessEntities, context);
                windowBusinessService.remove(idList, context);
            }
            //级联删除窗口事项
            WindowMatterQuery windowMatterQuery = new WindowMatterQuery();
            windowMatterQuery.setWindowIdList(Arrays.asList(ids));
            List<WindowMatterEntity> windowMatterEntities = windowMatterService.find(windowMatterQuery);
            log.info("remove windowMatterEntities size:{}",windowMatterEntities.size());
            if (!ObjectUtils.isEmpty(windowMatterEntities)) {
                Long[] idList = windowMatterEntities.stream().map(item -> item.getId()).toArray(Long[]::new);
                windowMatterService.remove(idList, context);
                //windowMatterService.removeList(windowMatterEntities, context);
            }
        }
        Arrays.asList(ids).forEach(id -> {
            pushChangeMsg(id);
        });

        super.removeAfter(ids, context, result);
    }


    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateAfter(WindowEntity entity, Context context) throws AppException {
        super.updateAfter(entity, context);
        pushChangeMsg(entity.getId());
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void saveAfter(WindowEntity entity, Context context) throws AppException {
        super.saveAfter(entity, context);
        pushChangeMsg(entity.getId());
    }

    private void pushChangeMsg(Long windowId) {
       // log.info("pushChangeMsg:{}", JSON.toJSONString(windowId));
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://172.15.28.116:8090");
        phpUrl += "/api/window/winNameChange";
        HashMap<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("windowid", windowId);
        paramsMap.put("typeinfo", 1);

        phpUrl += "/api/window/winNameChange";
        String resp = null;
        try {
            Map<String, String> header = new HashMap<>();
            header.put(HEADER_CONTENT_TYPE, "application/json");
            log.info("\n thirdPartyUrl=>{} \n reqbody=>{} ", phpUrl, JSON.toJSONString(paramsMap));
            resp = HttpUtil.doPost(phpUrl, header, JSON.toJSONString(paramsMap, SerializerFeature.WriteMapNullValue));
            log.info("thirdPartyUrl =>resp:{}", resp);
        } catch (Exception e) {
            log.error("异常:", e);
        }
    }
}